/*
 *	Video for Linux API plus additional API specifically for Corel Netwinder
 */
#ifndef _VIDEODEV_H
#define _VIDEODEV_H

#define V4L_MAJOR_VERSION  0
#define V4L_MINOR_VERSION  6

#ifdef __KERNEL__
#include <linux/types.h>
#include <linux/mm.h>
#endif

/*
 *	I O C T L S   F O R   V I D E O   C A P T U R E
 */
#define VIDIOC_IOC_MAGIC  'v'			/* magic number */
#define VIDIOCGCAP			_IOR(VIDIOC_IOC_MAGIC,	1,	struct video_capability)
#define VIDIOCGFBUF			_IOR(VIDIOC_IOC_MAGIC,	2,	struct video_buffer)
#define VIDIOCSFBUF			_IOW(VIDIOC_IOC_MAGIC,	3,	struct video_buffer)
#define VIDIOCGWIN			_IOR(VIDIOC_IOC_MAGIC,	4,	struct video_window)
#define VIDIOCSWIN			_IOWR(VIDIOC_IOC_MAGIC,	5,	struct video_window)
#define VIDIOCSCGWIN		_IOR(VIDIOC_IOC_MAGIC,	6,	struct video_capture)
#define VIDIOCSCSWIN		_IOWR(VIDIOC_IOC_MAGIC,	7,	struct video_capture)

#define VIDIOCCAPTURE		_IOW(VIDIOC_IOC_MAGIC,	8,	int)

#define VIDIOCGCHAN			_IOWR(VIDIOC_IOC_MAGIC,	9,	struct video_channel)
#define VIDIOCSCHAN			_IOW(VIDIOC_IOC_MAGIC,	10, int)
#define VIDIOCGPICT			_IOR(VIDIOC_IOC_MAGIC,	11, struct video_picture)
#define VIDIOCSPICT			_IOWR(VIDIOC_IOC_MAGIC,	12, struct video_picture)
#define VIDIOCGTUNER		_IOWR(VIDIOC_IOC_MAGIC, 13, struct video_tuner)	
#define VIDIOCSTUNER		_IOW(VIDIOC_IOC_MAGIC, 	14, int)
#define VIDIOCGFREQ			_IOR(VIDIOC_IOC_MAGIC,	15, __u32)
#define VIDIOCSFREQ			_IOW(VIDIOC_IOC_MAGIC, 	16, __u32) 
#define VIDIOCGAUDIO		_IOR(VIDIOC_IOC_MAGIC, 	17, struct video_audio)
#define VIDIOCSAUDIO		_IOW(VIDIOC_IOC_MAGIC, 	18, struct video_audio)
#define VIDIOCGMBUF			_IOR(VIDIOC_IOC_MAGIC,	19, struct video_mbuf)
#define VIDIOCMCAPTURE	_IOWR(VIDIOC_IOC_MAGIC,	20, int)
#define VIDIOCSYNC			_IOW(VIDIOC_IOC_MAGIC,	21, int)
#define VIDIOCGUNIT			_IOR(VIDIOC_IOC_MAGIC,	22, struct video_unit)

#define BASE_VIDIOC_PRIVATE 192   /* 192-255 are private */

/* 
 *	Additional Ioctls calls added for Corel Netwinder 
 */
#define VIDIOC_COREL_CFREEZEONOFF	_IOW(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+0, int)
#define VIDIOC_COREL_MIRRORX			_IOW(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+1, int)
#define VIDIOC_COREL_MIRRORY			_IOW(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+2, int)
#define VIDIOC_COREL_GDWIN				_IOR(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+3, struct display_window)
#define VIDIOC_COREL_SDWIN				_IOWR(VIDIOC_IOC_MAGIC,BASE_VIDIOC_PRIVATE+4, struct display_window)
#define VIDIOC_COREL_PREVIEW			_IOW(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+5, int)
#define VIDIOC_COREL_GPIP					_IOR(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+6, struct video_pip)
#define VIDIOC_COREL_SPIP					_IOWR(VIDIOC_IOC_MAGIC,BASE_VIDIOC_PRIVATE+7, struct video_pip)
#define VIDIOC_COREL_PIPONOFF			_IOW(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+8, int)
#define VIDIOC_COREL_GICON				_IOR(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+9, struct video_icon)
#define VIDIOC_COREL_SICON				_IOWR(VIDIOC_IOC_MAGIC,BASE_VIDIOC_PRIVATE+10,struct video_icon)
#define VIDIOC_COREL_ICONSELECT		_IOW(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+11,int)
#define VIDIOC_COREL_ICONONOFF		_IOW(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+12,int)
#define VIDIOC_COREL_GDFREEZE			_IOR(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+13,struct video_freeze)
#define VIDIOC_COREL_SDFREEZE			_IOWR(VIDIOC_IOC_MAGIC,BASE_VIDIOC_PRIVATE+14,struct video_freeze)
#define VIDIOC_COREL_DFREEZEONOFF	_IOW(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+15,int)
#define VIDIOC_COREL_GSHIFT				_IOR(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+16,struct video_shift)
#define VIDIOC_COREL_SSHIFT				_IOWR(VIDIOC_IOC_MAGIC,BASE_VIDIOC_PRIVATE+17,struct video_shift)
#define VIDIOC_COREL_SHIFTONOFF		_IOW(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+18,int)
/*
 *	X server private calls 
 */
#define VIDIOC_COREL_READONOFF				_IOW(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+19, int)
#define VIDIOC_COREL_S_SCREENOFFSET		_IOW(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+20, struct video_screen_params)
#define VIDIOC_COREL_G_SCREENOFFSET		_IOR(VIDIOC_IOC_MAGIC, BASE_VIDIOC_PRIVATE+21, struct video_screen_params)


/*
 *	D R I V E R   C A P A B I L I T I E S
 */

/*
 * 	Video Capability
 */
struct video_capability 
{
	char name[32];	/* Cannonical name for this interface */	
	int type;   		/* Feature flags, see below */
	int channels;		/* Num tv/radio channels if appropriate */
	int audios;			/* Num audio devices */
	int maxwidth;		/* Maximum capture width in pixels */
	int maxheight;	/* Maximum capture height in pixels */
	int minwidth;		/* Minimum capture width in pixels */
	int minheight;	/* Minimum capture height in pixels */
};

/* Flags for 'type' field */
#define VID_TYPE_CAPTURE			0x0001  /* 	Can capture to memory */
#define VID_TYPE_TUNER				0x0002	/* 	Has a tuner of some form */
#define VID_TYPE_TELETEXT			0x0004	/* 	Has teletext capability */
#define VID_TYPE_OVERLAY			0x0008	/*	Can overlay its image onto the frame
																					buffer */
#define VID_TYPE_CHROMAKEY		0x0010	/*	Overlay is Chromakeyed */
#define VID_TYPE_CLIPPING			0x0020	/*	Overlay clipping is supported */
#define VID_TYPE_FRAMERAM			0x0040	/*	Overlay overwrites frame buffer 
																					memory */
#define VID_TYPE_SCALES				0x0080	/*	The hardware supports image scaling */
#define VID_TYPE_MONOCHROME		0x0100  /* 	Image capture is grey scale only */
#define VID_TYPE_SUBCAPTURE		0x0200	/*	Capture can be of only part of the
																					image */
/*
 *	Frame Buffer	(This is used to communicate with X server)
 */
struct video_buffer
{
	void* base;					/* Starting address of the available memory region */
	int height;					/* Height x Width will indicate the available memory 
													size for the use of the video driver */
	int width;					
	int depth;					/* Depth of the frame buffer (16 or 24) */
	int bytesperline;		/* Number of bytes of memory between the start of two
													adjacent lines */
};

/*
 *	Capture Windows	(description of normal capture area)
 */
struct clip_rectangle		/* Specifying overlay clipping area */
{
	int x;				/* X co-ordinate of rectangle to skip */
	int y;				/* Y co-ordinate of rectangle to skip */
	int width;		/* Width of rectangle to skip  */
	int height;		/* Height of rectangle to skip */
};

struct video_window
{
	int x;								/* X co-ordinate specified in X windows format. */
	int y;								/* Y co-ordinate specified in X windows format. */
	int width;						/* The width of the image capture. */
	int height;						/* The height of the image capture. */
	__u32 chromakey;			/* A host order RGB32 value for the chroma key. */
	int flags;						/* Additional capture flags. (NONE) */
	struct clip_rectangle* clips;			/* Overlay clipping NOT supported */
	int clipcount;										/* Overlay clipping NOT supported */
};

/*
 *	Sub-capture Windows (description of sub-capture area)
 */
struct video_capture
{
	int x;								/* X co-ordinate of source rectangle to grab */
	int y;								/* Y co-ordinate of source rectangle to grab */
	int width;						/* Width of source rectangle to grab */
	int height;						/* Height of source rectangle to grab */
	int decimation;				/* e.g. Decimation by n means, returning every other
														n pixel. If n=5, returning 1,5,10,15,20.... 
														(NOT supported) */
	int flags;						/* Flag setting for grabbing (NOT supported) */ 
};

/* Flags */
#define VIDEO_CAPTURE_ODD			0x0001
#define VIDEO_CAPTURE_EVEN		0x0002

/*
 *	Video Sources
 */
struct video_channel
{
	int channel;			/* The channel number */
	char name[32];		/* The input name - preferably reflecting the label on
												the card input itself */
	int tuners;				/* Number of tuners for this input */
	int flags;				/* Properties the tuner has */
	int type;					/* Input type (if known) */
	int norm;					/* The norm for this channel */
};

/* Flags */
#define VIDEO_VC_TUNER			0x0001	/* Channel has tuners. */
#define VIDEO_VC_AUDIO			0x0002	/* Channel has audio. */
#define VIDEO_VC_NORM				0x0004	/* Channel has norm setting. */

/* Types */
#define VIDEO_TYPE_TV				0x0001	/* The input is a TV input. */
#define VIDEO_TYPE_CAMERA		0x0002	/* The input is a camera. */

/*
 *	Image properties of the picture
 */
struct video_picture
{
	__u16 brightness;			/* All values except for the palette type are scaled
														between 0 - 65535 */
	__u16 hue;						/* (Tint) colour only */
	__u16 colour;					/* (Saturation) colour only */
	__u16 contrast;
	__u16 whiteness;			/* The whiteness (greyscale only) (Not Used) */
	__u16 depth;					/* The capture depth (may need to match the	
														frame buffer depth) (Get ONLY) */
	int palette;					/* Reports the palette that should be used
														for this image */
};

/* Palette */
#define VIDEO_PALETTE_GREY		0x0001	/* 8 bpp  Linear intensity grey scale 
																				 				(255 is brightest). */
#define VIDEO_PALETTE_HI240		0x0002	/* 8 bpp  The BT848 8bit colour cube. */
#define VIDEO_PALETTE_RGB565	0x0004	/* 16 bpp RGB-5-6-5 packed RGB format. */
#define VIDEO_PALETTE_RGB555	0x0008	/* 16 bpp RGB-5-5-5 packed RGB format,
																								high bit undefined. */ 
#define VIDEO_PALETTE_RGB24		0x0010	/* 24 bpp RGB-8-8-8 packed into 24-bit 
																								words. B is at byte 0. */
#define VIDEO_PALETTE_RGB32		0x0020 	/* 32 bpp RGB-8-8-8 packed into 32-bit
																								words. B is at byte 0. Byte 3
																								is undefined. */
#define VIDEO_PALETTE_YUYV		0x0080 	/* 16 bpp YUV4:2:2, 8bits/component. 
																								Byte0 = Y0, Byte1 = U01, 
																								Byte2 = Y1, Byte3 = V01, etc */
#define VIDEO_PALETTE_UYVY		0x0100	/* 16 bpp Same as YUYV, except U-Y-V-Y
																								byte order */
#define VIDEO_PALETTE_YUV420	0x0200	/* 12 bpp YUV420 capture */
#define VIDEO_PALETTE_YUV411	0x0400	/* 12 bpp YUV411 capture */ 
#define VIDEO_PALETTE_YUV422	0x0040 	/* 8 bpp  Video style YUV422-8bit packed 
																								4bits Y 2 bits U 2 bits V */
#define VIDEO_PALETTE_RAW			0x0800	/*  */
#define VIDEO_PALETTE_YUV422P	0x1000	/*  */
#define VIDEO_PALETTE_YUV411P	0x2000	/*  */

/*
 *	Tuning (NOT supported)
 */
struct video_tuner
{
	int tuner;										/* The number of the tuner */
	char name[32];								/* Cannonical name for this tuner 
																		(e.g. FM/AM/TV) */
	__u32 rangelow;								/* Lowest tunable frequency */
	__u32 rangehigh;							/* Highest tunable frequency */
	int flags;										/* Flags describing the tuner */
	int mode;											/* The video signal mode if relevant */
	unsigned short signal;				/* Signal strength if known - between 0-65535 */
};

/* Flags */
#define VIDEO_TUNER_PAL					0x0001		/* PAL tuning is supported */
#define VIDEO_TUNER_NTSC				0x0002		/* NTSC tuning is supported */
#define VIDEO_TUNER_SECAM				0x0004		/* SECAM tuning is supported */
#define VIDEO_TUNER_LOW					0x0010		/* Frequency is in a lower range */
#define VIDEO_TUNER_NORM				0x0020		/* The norm for this tuner is 
																							settable */
#define VIDEO_TUNER_STEREO_ON		0x0040		/* The tuner is seeing stereo audio */

/* Modes */
#define VIDEO_MODE_PAL					0x0001		/* The tuner is in PAL mode */
#define VIDEO_MODE_NTSC					0x0002		/* The tuner is in NTSC mode */
#define VIDEO_MODE_SECAM				0x0004		/* The tuner is in SECAM mode */
#define VIDEO_MODE_AUTO					0x0010		/* The tuner auto switches, or mode
																							does not apply */
/*
 *	Audio (NOT supported)
 */
struct video_audio
{
	int audio;			/* The channel number */
	int volume;			/* The volume level */
	int bass;				/* The bass level */
	int treble;			/* The treble level */
	int flags;			/* Flags describing the audio channel */
	char name[32];	/* Canonical name for the audio input */
	int mode;				/* The mode the audio input is in */
	int balance;		/* The left/right balance */
	int step;				/* Actual step used by the hardware */
};

/* Flags */
#define VIDEO_AUDIO_MUTE			0x0001		/* The audio is muted */
#define VIDEO_AUDIO_MUTABLE		0x0002		/* Audio muting is supported */
#define VIDEO_AUDIO_VOLUME		0x0004		/* The volume is controllable */
#define VIDEO_AUDIO_BASS			0x0010		/* The bass is controllable */
#define VIDEO_AUDIO_TREBLE		0x0020		/* The treble is controllable */
#define VIDEO_AUDIO_BALANCE		0x0040		/* The balance is controllable */

/* Modes */
#define VIDEO_SOUND_MODE			0x0001		/* Mono signal */
#define VIDEO_SOUND_STEREO		0x0002		/* Stereo signal (NICAM for TV) */
#define VIDEO_SOUND_LANG1			0x0004		/* European TV alternate language 1 */
#define VIDEO_SOUND_LANG2			0x0010		/* European TV alternate language 2 */

/*
 *	Query mmap info (NOT supported)
 */ 
struct video_mbuf
{
	int size;									/* The number of bytes to map */
	int frames;								/* The number of frames (max is 1 frame) */
	unsigned long offsets;		/* The offset of each frame (only one here) */
};

/*
 *	Unit number of associated devices
 */
struct video_unit
{
	int video;		/* Video capture device */
	int vbi;			/* VBI capture device */
	int radio;		/* Radio device */
	int audio;		/* Audio mixer */
	int teletext;	/* Teletext device */
};


/*
 *	S T R U C T U R E S   N E E D E D   F O R   N E W   I O C T L   C A L L S 
 */

/*
 *	Video shifting
 */
struct video_shift
{
	int x;							/* Coordinate of upper left corner of selected area */
	int y;
	int width;					/* Dimension of the selected area */
	int height;
	int src;						/* Which video window does the data provided from...
													0-Main, 1-V2, 2-X2 */
};

/*
 *	Video display freezing
 */
struct video_freeze
{
	int x;							/* Coordinate of upper left corner of selected area */
	int y;
	int width;					/* Dimension of the selected area */
	int height;
	int type:1;					/* 0 - selected area is freezed, surroundings moving */
											/* 1 - selected area is moving, surroundings freezed */
};

/*
 *	Iconic Video
 */
struct video_icon
{
  int total_num;      /* Total number of icon video, maximum is 6 */
	int width;					/* The dimension of each icon, */
  int height;					/*  maximum is 88x72
                          minimum is 40x32  */
  int src;            /* Which video window does the data provided from...
                          0-Main, 1-V2, 2-X2 */
};

/*
 *	Picture in Picture (PIP)
 */
struct video_pip
{
	int x;							/* Coordinate of upper left corner of selected area */
	int y;
	int width;					/* Dimension of selected area */
	int height;
	int type:1;					/* type = 0 (Hardware PIP), type = 1 (Software PIP) */
	int order:1;				/* order = 0 (Background), order = 1 (Foreground) */
};

/* 
 * Set/Get current display window parameters 
 */
struct display_window
{
	struct video_window dest_win;
	int									UseChromaKey:1;	/* Using chromakey to overlay image? */
	int									SrcType:1;			/* 0 - Local data, 1 - Remote data */
	int									SrcPalette;			/* Pixel format of the source data */
	int									SrcWidth;				/* Dimension of video data */
	int									SrcHeight;
                                    	/* 	Note: if SrcType is 0, 
																					SrcPalette, SrcWidth and SrcHeight
                                        	are ignored.
																			*/
};

#endif /* _VIDEODEV_H */
